package ExtUtils::Typemap::Basic;
use strict;
use warnings;
use ExtUtils::Typemaps;
use ExtUtils::Typemaps::Basic;

our $VERSION = '1.05';

our @ISA = qw(ExtUtils::Typemaps::Basic);

=head1 NAME

ExtUtils::Typemap::Basic - A set of typemaps for simple types

=head1 SYNOPSIS

This module had to be renamed. Use L<ExtUtils::Typemaps::Basic> instead!

=head1 DESCRIPTION

This module had to be renamed. Use L<ExtUtils::Typemaps::Basic> instead!

=head1 SEE ALSO

L<ExtUtils::Typemaps::Basic>

L<ExtUtils::Typemap>, L<ExtUtils::Typemaps>

=head1 AUTHOR

Steffen Mueller <smueller@cpan.org>

=head1 COPYRIGHT AND LICENSE

Copyright 2010, 2011, 2012, 2013 by Steffen Mueller

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

1;
